/*

	Filename:   CDfinder.prg
   Version:    1.03
   Date:       1999/03/08

   For:        VdBase 7.x / Win95, Win98, Win98SE

   Author:     Ronnie MacGregor
   Contact:    RBMacG@High-Lugtonridge.co.uk

	
   Revisions:  

   1999/03/08  Coded for FrameCat application.
 
   1999/03/13  Check for no CDROM drive or faulty added.

   2000/01/19  Recoded to provide generic functionality.

   2000/02/22  Name of required CD parameter added.
               Validation of parameters added.
               PathChanger function written (generic automated version
               of SQLchanger written for FrameCat application)

   
   Description:
   ************

      CDfinder() is designed for applications which need to access
      tables stored on a CD. It is intended to be called by your
      application on startup to look for a CDROM Drive; look for the
      CD in that drive, and if required prompt the user to put the CD
      into the drive.

      CDfinder() will return the Drive Letter for the successfully 
      found CDROM Drive containing the correct CD, or will return
      the space character " " if it is unsuccessful.

      Having found the CD in the drive, and knowing the drive letter
      you can call PathChanger() to find and change all occurences of
      drive letters in your SQL or DMD file to that of the CD Drive,
      returning true or false depending on success.
      *** Use with care *** make sure you realise what this is doing
      to your SQL or DMD file before you use it.
      

   CDfinder() Syntax:
   ******************

      CDfinder(<cFile>[,<cName>][, oRef])

      <cFile>

         A String containing a file which will uniquely be found on
         the required CD

      <cName> (optional)

         A string containing a name by which the user will recognise
         the CD that is required

      <oRef> (optional)

         An object reference  to try to attach the
         custom property .CDdrive to.

      Example:

      Set Procedure to CDfinder.prg additive
      cCDdrive = CDfinder( "MyDir\MyFile.dbf","Visual dBase", form )

   
   PathChanger() Syntax:
   *********************

      PathChanger(<cFile>,<cDrive>)

      <cFile>

         A String containing the file name of the SQL or DMD
         to be changed

      <cDrive>

         The drive letter you wish to use in your SQL or DMD

      Example:

      Set Procedure to CDfinder.prg additive // if required
      cHasPathChanged = PathChanger( "MyDataMod.dmd", form.CDdrive )
   

   GetDrives() Syntax:
   *******************

      aDrives = GetDrives([< nType >])

      <nType>

         A number indicating the type of drive you wish to find
         from this list:

         0  The drive type cannot be determined.
         1  The root directory does not exist.
         2  The drive can be removed from the drive.
         3  The disk cannot be removed from the drive.
         4  The drive is a remote (network) drive.
         5  The drive is a CD-ROM drive.
         6  The drive is a RAM disk.


      Example:

      Set Procedure to CDfinder.prg additive // if required
      aDrives = GetDrives(5)
   
   
   Dependancies:
   *************

      None

   Note:
   *****

      CDfinder will validate and possibly change the parameters
      passed to it. If parameters are passed by reference then these
      changes will be reflected in the calling application. This may
      or may not be desired, and if it is not desired then the
      parameters should be passed by value by enclosing them in
      parenthesis.

   Acknowledgements:
   *****************

      GetDrives code adapted from the dUFLP version.

      Thanks to Romain Strieff, for snippets of information and help
      given in the dBase news groups.
             
*/


Function CDFinder(cFile,cCDname,oRef)
local cReturn, nChoice
//
// Check parameters
if empty(argvector(1))
   MsgBox("File name parameter not passed","Call Error")
   cReturn = " "
   return cReturn
else
   // strip off any drive letters etc
   cFile = substr(cFile,(at(":",cFile,1)+1),len(cFile))
   if substr(cFile,1,1) = "\"
      cFile = substr(cFile,2,len(cFile))
   endif
endif
if empty(argvector(2))
  cCDname = "required"
endif
//
// Find CDROM Drives
aCDdrives=Getdrives(5)//returns array containing all the CD drives
// Next 3 lines output the drives found in the results pane
// for n=1 to aCDdrives.size
//    ? aCDdrives[n]   
// next
if aCDdrives.size = 0
   MsgBox("CD ROM drive not found or faulty ","CD ROM drive error")
   cReturn = " "
else
   // Check object flagging parameter
   if not empty(argvector(3))
      try
         oRef.CDdrive = aCDdrives[1]         
      catch ( Exception e )
         msgbox("The calling program did not pass" + chr(13) + ;
                "a valid object reference parameter.","Call Error",16)
      endTry
   endif
   //
   //  Look for CD in drive
   nChoice = 0
   // Loop exit control is inside loop
   do while nChoice < 3  
      if not file(aCDdrives[1] + ":\" + cFile) // file only on this CD
	      nChoice = MsgBox("Please insert the " + cCDname ;
         + " CD into Drive " + aCDdrives[1],"CD ROM drive error",1)
         if nChoice = 1
            loop // Until correct CD or the user cancels
         endif
         if nChoice = 2
            cReturn = " "   //  User cancel
            exit           //  Exit loop
         endif
      endif
      cReturn = aCDdrives[1]
      exit // Exit loop if CD is found
   enddo 
endif
return cReturn


Function PathChanger(cFile, cDrive)
local cThisChar
// Check parameters
if empty(argvector(1))
   MsgBox("File Name parameter not passed","Call Error")
   return False
endif
if empty(argvector(2))
   MsgBox("Drive Letter parameter not passed","Call Error")
   return False
endif
// Find and change Drive Letters
fChange = new File()
try
   fChange.Open(cFile,"RW")
   do while not fChange.eof()
      cThisChar = fChange.Read(1)
      if cThisChar = ":"
         cThisChar = fChange.Read(1)
         if cThisChar = "\"
            fChange.Seek(-3,1)
            fChange.Write(cDrive)
            fChange.Seek(3,1)
         endif
      endif
   enddo
   fChange.Close()
catch(Exception e)
   return False
endtry
return  True


Function GetDrives(nType)
// Origin - dUFLP (via Romain in the newsgroups)
//
#define KERNELDLL  KERNEL32
#define WINAPI_A   "A"
//
if type("GetDriveType")# "FP"
   extern cInt GetDriveType(cstring);
   KERNELDLL from "GetDriveType"+WINAPI_A
endif
//
aDrives=new array()
for n=65 to (65+25)
   c=chr(n)
   if GetDriveType(c+":\") = nType
      aDrives.add(c)
   endif
next
return aDrives

//eop
